/*
 * expires.c: NSAPI SAF to set an HTTP expires header
 * v 1.0
 *
 * Version History:
 * v1.0: November 7, 2003
 *       Initial release - DzM (www.dzm.com)
 *
 *
 * Use:
 * ======================================================================
 * magnus.conf:
 * ------------
 *
 * Init fn="load-modules" shlib="/path/to/expires.so" funcs="set_expires"
 *
 * obj.conf
 * ------------
 *
 * PathCheck fn="set_expires" [options]
 *
 * Valid options are:
 *  offset=[seconds]  - Integer setting offset from NOW in seconds
 *  days=[days]       - Integer setting offset from NOW in days
 *  hours=[hours]     - Integer setting offset from NOW in hours
 *  minutes=[minutes] - Integer setting offset from NOW in minutes
 *  seconds=[seconds] - Integer setting offset from NOW in seconds
 *
 * All values may be positive (to set the expires header in the
 * future) or negative (to set the expires header in the past).
 *
 * All options are cumulative. days="2" hours="-12" and days="1"
 * hours="12" are the same as hours="36"
 *
 * All options are optional. Is none are present the expires header is
 * set to NOW (e.g. the the expires header is equal to the date
 * header).
 *
 * Examples:
 * ======================================================================
 * --> 1) Set the expires header for each request the server takes to
 * expire in six hours:
 * 
 * <Object default>
 * ...
 * PathCheck fn="set_expires"
 *           hours="6"
 * ...
 *
 *
 * --> 2) Set the expires header to 24 hours in the future for requests
 * from a specified directory:
 *
 * <Object default>
 * ...
 * NameTrans fn="assign-name"
 *           from="/uri"
 *           name="one_day_expire"
 * ...
 * </Object>
 * <Object name="one_day_expire">
 * PathCheck fn="set_expires"
 *           days="1"
 * </Object>
 *
 *
 * --> 3) Set the expires header to five minutes ago
 *
 * PathCheck fn="set_expires"
 *           minutes="-5"
 * 
 *
 * Thanks:
 * ======================================================================
 * Uses an idea from Alex Stankevich
 * Assistance was given by
 *   Rob Crittenden (www.greyoak.com)
 *   Chris Elving (www.elving.com)
 *
 * DzM
 */

/************************* No User Servicable Parts **********************/

/* Standard NSAPI headers */
#include "nsapi.h"
#include <time.h>

/* Deal with Windows */
#ifdef XP_WIN32
#define NSAPI_PUBLIC __declspec(dllexport)
#else /* !XP_WIN32 */
#define NSAPI_PUBLIC
#endif /* !XP_WIN32 */

/* Just plain crazy. */
#ifdef __cplusplus
extern "C"
#endif

int set_expires( pblock *pb, Session *sn, Request *rq ){
  int return_value = REQ_NOACTION;
  char expdate[50]; /* A tidy buffer to hold our new header in */
  int res;
  time_t t_expiration;
  struct tm tm_expiration;

  int offset=0; /* Our default offset is zero seconds. */ 
  char *o = pblock_findval("offset", pb); /* Acquire pointers to the possible options */
  char *d = pblock_findval("days", pb);
  char *h = pblock_findval("hours", pb);
  char *m = pblock_findval("minutes", pb);
  char *s = pblock_findval("seconds", pb);

  if (o) /* If "offset" option was used, acquire it - already in seconds */
    offset=atoi(o);
  if (d) /* If "day" option was used, acquire it and convert to seconds */
    offset+=(atoi(d) * 86400);
  if (h) /* If "hours" option was used, acquire it and convert to seconds */
    offset+=(atoi(h) * 3600);
  if (m) /* If "minutes" option was used, acquire it and convert to seconds */
    offset+=(atoi(m) * 60);
  if (s) /* If "seconds" option was used, acquire it - already in seconds */
    offset+=atoi(s);

  time(&t_expiration); /* Acquire the time NOW */
  t_expiration += offset; /* Adjust it by the specified value */

  system_gmtime(&t_expiration, &tm_expiration); /* Adjust everything to reflect GMT */
  util_strftime(expdate,HTTP_DATE_FMT,&tm_expiration); /* Format the adjusted time and store it */

  param_free(pblock_remove("expires",rq->srvhdrs)); /* Flush any existing expires header */
  pblock_nvinsert("expires",expdate,rq->srvhdrs); /* Insert our new header */

  return return_value; /* Proceed with request processing */
}
